"use strict";
Object.defineProperty(exports, Symbol.toStringTag, { value: "Module" });
const lit = require("lit");
const decorators_js = require("lit/decorators.js");
const ref_js = require("lit/directives/ref.js");
const uuid = require("uuid");
const remoteManager = require("./remote-manager.cjs");
const utils = require("./utils.cjs");
var __defProp = Object.defineProperty;
var __getOwnPropDesc = Object.getOwnPropertyDescriptor;
var __decorateClass = (decorators, target, key, kind) => {
  var result = kind > 1 ? void 0 : kind ? __getOwnPropDesc(target, key) : target;
  for (var i = decorators.length - 1, decorator; i >= 0; i--)
    if (decorator = decorators[i])
      result = (kind ? decorator(target, key, result) : decorator(result)) || result;
  if (kind && result) __defProp(target, key, result);
  return result;
};
const TAG_NAME = "engineai-web-component";
exports.EngineAIWebComponentElement = class EngineAIWebComponentElement extends lit.LitElement {
  constructor() {
    super();
    this.src = "";
    this.options = void 0;
    this.token = void 0;
    this.resizable = true;
    this.onStateChange = null;
    this.instanceId = uuid.v4();
    this.remoteManager = new remoteManager.RemoteManager(this);
    this.isLoaded = false;
    this.contentHeightChangeHandler = (ev) => {
      var _a;
      if (!this.resizable || ev.action.type !== "content-height-change") {
        return;
      }
      this.style.minHeight = ((_a = ev.action.data) == null ? void 0 : _a.height) != null ? `${ev.action.data.height}px` : "";
    };
    this.addEventListener("state-change", (ev) => {
      var _a;
      return (_a = this.onStateChange) == null ? void 0 : _a.call(this, ev);
    });
    this.addEventListener("state-change", this.contentHeightChangeHandler);
  }
  /**
   * @param property - camel case property
   * @returns attribute name using kebab case (hyphen)
   */
  static attributeNameForProperty(property2) {
    return property2.replace(/[A-Z]/g, (m) => `-${m.toLowerCase()}`);
  }
  updated(changedProperties) {
    if (!this.isLoaded) {
      return;
    }
    if (changedProperties.has("src")) {
      this.onSrcUpdate(changedProperties.get("src"));
    }
    if (changedProperties.has("token")) {
      this.onTokenUpdate(changedProperties.get("token"));
    }
    if (changedProperties.has("options")) {
      this.onOptionsUpdate();
    }
  }
  onSrcUpdate(oldValue) {
    if (this.src) {
      const currentUrl = new URL(this.src);
      const oldUrl = new URL(oldValue);
      if (currentUrl.origin === oldUrl.origin) {
        this.remoteManager.dispatch("location-change", {
          path: currentUrl.pathname + currentUrl.search + currentUrl.hash
        });
      } else {
        this.iframe.src = this.src;
      }
    }
  }
  onTokenUpdate(oldValue) {
    if (this.token) {
      this.remoteManager.dispatch("login", { token: this.token });
    } else if (oldValue) {
      this.remoteManager.dispatch("logout");
    }
  }
  onOptionsUpdate() {
    if (this.options) {
      this.remoteManager.dispatch("options", this.options);
    }
  }
  disconnectedCallback() {
    super.disconnectedCallback();
    this.remoteManager.destroy();
  }
  render() {
    if (!this.src) {
      return null;
    }
    return lit.html`
      <iframe
        ${ref_js.ref(this.iframeRefCallback)}
        title="EngineAI Web Component"
        frameborder="0"
      ></iframe>
    `;
  }
  /**
   * ref callback called with element on mount or null on unmount
   */
  iframeRefCallback(el) {
    this.iframe = el;
    if (!el) {
      this.remoteManager.disconnect();
      this.isLoaded = false;
      return;
    }
    const url = new URL(this.src);
    const queryParams = new URLSearchParams(url.search);
    queryParams.set("web-component-version", remoteManager.version);
    queryParams.set("web-component-instance", this.instanceId);
    queryParams.set("web-component-resizable", this.resizable.toString());
    url.search = queryParams.toString();
    el.src = url.href;
    el.onload = () => {
      this.remoteManager.connect(el.contentWindow);
      this.isLoaded = true;
      const changedProperties = Object.entries({
        token: void 0,
        options: void 0
      });
      this.updated(new Map(changedProperties));
    };
  }
  addEventListener(type, listener, options) {
    super.addEventListener(
      type,
      listener,
      options
    );
  }
  removeEventListener(type, listener, options) {
    super.removeEventListener(
      type,
      listener,
      options
    );
  }
  dispatchEvent(ev) {
    return super.dispatchEvent(ev);
  }
};
exports.EngineAIWebComponentElement.tagName = TAG_NAME;
exports.EngineAIWebComponentElement.shadowRootOptions = {
  mode: "open"
};
exports.EngineAIWebComponentElement.styles = lit.css`
    iframe {
      height: 100%;
      width: 100%;
    }
  `;
__decorateClass([
  decorators_js.property({ type: String, reflect: true })
], exports.EngineAIWebComponentElement.prototype, "src", 2);
__decorateClass([
  decorators_js.property({ type: String, reflect: true })
], exports.EngineAIWebComponentElement.prototype, "options", 2);
__decorateClass([
  decorators_js.property({ type: String, reflect: true })
], exports.EngineAIWebComponentElement.prototype, "token", 2);
__decorateClass([
  decorators_js.property({ type: Boolean, reflect: true })
], exports.EngineAIWebComponentElement.prototype, "resizable", 2);
__decorateClass([
  decorators_js.property({
    converter: utils.AttributeEventConverter,
    reflect: false
  })
], exports.EngineAIWebComponentElement.prototype, "onStateChange", 2);
exports.EngineAIWebComponentElement = __decorateClass([
  decorators_js.customElement(TAG_NAME)
], exports.EngineAIWebComponentElement);
