import { EngineAIWebComponentStateChangeEvent } from "./events.js";
const name = "@engineai-dev/web-component";
const version = "1.1.0";
class RemoteManager {
  constructor(element) {
    this.element = element;
    this.abortController = new AbortController();
    this.target = null;
    window.addEventListener(
      "message",
      (e) => {
        if (e.data.instance !== this.element.instanceId) {
          return;
        }
        const action = e.data.action;
        this.element.dispatchEvent(
          new EngineAIWebComponentStateChangeEvent(action)
        );
      },
      {
        signal: this.abortController.signal
      }
    );
  }
  /**
   * client to dispatch actions on the remote endpoint
   */
  dispatch(type, data) {
    if (!this.target) {
      throw new Error("call connect() before using remote");
    }
    this.target.postMessage(
      {
        instance: this.element.instanceId,
        source: name,
        action: {
          type,
          data
        }
      },
      "*"
    );
  }
  connect(target) {
    this.target = target;
  }
  disconnect() {
    this.target = null;
  }
  destroy() {
    this.abortController.abort();
  }
}
export {
  RemoteManager,
  version as v
};
